# -------------------------------------------------------------------------------------------------------------
# Dieses Skript wurde basierend auf einer Idee und den Anforderungen von spakk (insanelymac.com) entwickelt.
#
# Das Ziel dieses Skripts ist es, eine zuverlässige und anpassbare Methode zur Erstellung eines 1:1-Klons
# einer APFS-Partition inklusive einer optionalen Klonierung der EFI-Partition bereitzustellen.
#
# Lizenz:
# Dieses Skript steht frei zur Verfügung und darf gemäß den Prinzipien der freien Software genutzt,
# verändert und weiterverbreitet werden. Änderungen können ohne Einschränkungen vorgenommen werden,
# jedoch sind der Autor und Mitwirkende nicht verantwortlich für eventuelle Schäden oder Datenverluste.
#
# Haftungsausschluss:
# Dieses Skript wird ohne jegliche Garantie bereitgestellt. Die Nutzung erfolgt auf eigenes Risiko.
# Es wird dringend empfohlen, eine Sicherungskopie Ihrer Daten zu erstellen, bevor das Skript verwendet wird.
#
# Hinweise:
# Bei Fragen, Feedback oder weiteren Verbesserungsvorschlägen, zögern Sie nicht, sich mit der Gemeinschaft
# der Entwickler und Benutzer auszutauschen.
#
# Viel Erfolg und Freude bei der Nutzung dieses Skripts!
# -------------------------------------------------------------------------------------------------------------



LOGFILE="clone_log_$(date +%Y%m%d_%H%M%S).txt"
echo "Start des Klonprozesses am $(date)" > "$LOGFILE"

# Funktion zur Anzeige von Partitionen
list_partitions() {
    echo "\nVerfügbare Partitionen:"
    diskutil list | grep -E "(APFS Volume|EFI)"
}

# Funktion zur Benutzereingabe
get_user_input() {
    local prompt="$1"
    local input
    read -p "$prompt" input
    echo "$input"
}

# Partitionen auflisten
list_partitions

# Benutzer nach Quell- und Ziel-Partitionen fragen
SOURCE_DEVICE=$(get_user_input "Bitte geben Sie die Quell-Partition (z. B. disk1s1) ein: ")
TARGET_DEVICE=$(get_user_input "Bitte geben Sie die Ziel-Partition (z. B. disk2s2) ein: ")

# Quell- und Zielgeräte überprüfen
if ! diskutil info "$SOURCE_DEVICE" > /dev/null 2>&1; then
    echo "Fehler: Quell-Partition '$SOURCE_DEVICE' wurde nicht gefunden." | tee -a "$LOGFILE"
    exit 1
fi

if ! diskutil info "$TARGET_DEVICE" > /dev/null 2>&1; then
    echo "Fehler: Ziel-Partition '$TARGET_DEVICE' wurde nicht gefunden." | tee -a "$LOGFILE"
    exit 1
fi

echo "Quell-Partition: $SOURCE_DEVICE" | tee -a "$LOGFILE"
echo "Ziel-Partition: $TARGET_DEVICE" | tee -a "$LOGFILE"

# Prüfen, ob eine EFI-Partition auf dem Zielgerät existiert
TARGET_DISK=$(echo "$TARGET_DEVICE" | sed 's/s[0-9]*$//')
EFI_PARTITION=$(diskutil list "$TARGET_DISK" | grep "EFI" | awk '{print $6}')

if [ -n "$EFI_PARTITION" ]; then
    echo "WARNUNG: Eine EFI-Partition wurde auf dem Zielgerät gefunden." | tee -a "$LOGFILE"
    read -p "Möchten Sie die EFI-Partition überschreiben? (ja/nein): " overwrite_efi
    if [[ "$overwrite_efi" != "ja" ]]; then
        echo "Der Klonprozess wird ohne Überschreiben der EFI-Partition fortgesetzt." | tee -a "$LOGFILE"
    else
        echo "Die EFI-Partition wird überschrieben." | tee -a "$LOGFILE"
    fi
fi

# Ziel-Partition aushängen und formatieren
echo "Formatieren der Ziel-Partition als APFS..." | tee -a "$LOGFILE"
diskutil unmountDisk force "$TARGET_DISK"
diskutil eraseDisk APFS "Klon-Ziel" "$TARGET_DISK" | tee -a "$LOGFILE"

# Quell-Partition klonen
echo "Klonen der APFS-Partition..." | tee -a "$LOGFILE"
sudo asr restore --source "/dev/$SOURCE_DEVICE" --target "/dev/$TARGET_DEVICE" --erase --noprompt --noverify | tee -a "$LOGFILE"

# EFI-Partition klonen, falls ausgewählt
if [[ "$overwrite_efi" == "ja" ]]; then
    echo "Klonen der EFI-Partition..." | tee -a "$LOGFILE"
    SOURCE_DISK=$(echo "$SOURCE_DEVICE" | sed 's/s[0-9]*$//')
    SOURCE_EFI=$(diskutil list "$SOURCE_DISK" | grep "EFI" | awk '{print $6}')
    TARGET_EFI=$(diskutil list "$TARGET_DISK" | grep "EFI" | awk '{print $6}')

    if [ -n "$SOURCE_EFI" ] && [ -n "$TARGET_EFI" ]; then
        sudo dd if="/dev/$SOURCE_EFI" of="/dev/$TARGET_EFI" bs=512 | tee -a "$LOGFILE"
        echo "EFI-Partition erfolgreich geklont." | tee -a "$LOGFILE"
    else
        echo "Fehler beim Finden der EFI-Partitionen." | tee -a "$LOGFILE"
    fi
fi

# Bootfähigkeit sicherstellen
echo "Stellen der Bootfähigkeit sicher..." | tee -a "$LOGFILE"
sudo bless --folder "/Volumes/Klon-Ziel/System/Library/CoreServices" --bootefi --setBoot | tee -a "$LOGFILE"

echo "Klonprozess abgeschlossen. Protokoll gespeichert in '$LOGFILE'." | tee -a "$LOGFILE"

# -------------------------------------------------------------------------------------------------------------
# Hinweis:
# Nachdem der Klonvorgang erfolgreich abgeschlossen ist, starten Sie Ihren Hackintosh neu,
# um den Klon einschließlich der EFI-Partition zu testen.
#
# Wählen Sie während des Neustarts im Boot-Menü die zuvor geklonte Festplatte aus.
#
# Überprüfen Sie die Funktionalität des Systems sorgfältig, insbesondere bei der Nutzung
# der EFI-Partition, um sicherzustellen, dass der Klon ordnungsgemäß funktioniert.
#
# Viel Erfolg!
# -------------------------------------------------------------------------------------------------------------
